package internal

import (
	"context"
	"fmt"
	"net/http"
	"time"

	"github.com/gorilla/mux"
	"k8s.io/apimachinery/pkg/api/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/client-go/kubernetes"
	"sigs.k8s.io/controller-runtime/pkg/healthz"
)

func NewServer(ctx context.Context, config Config, client *kubernetes.Clientset, manager *Manager) http.Server {
	router := mux.NewRouter()
	handler := healthz.Handler{
		Checks: map[string]healthz.Checker{
			"ping": manager.Check,
		},
	}
	router.Handle("/healthz", http.StripPrefix("/healthz", &handler))
	router.HandleFunc("/graceful-shutdown/keydb", gracefulShutdownKeyDBHandler(ctx, config, client))
	router.HandleFunc("/graceful-shutdown/manager", gracefulShutdownManagerHandler(manager))
	return http.Server{
		Addr:    ":80",
		Handler: router,
	}
}

func gracefulShutdownKeyDBHandler(ctx context.Context, config Config, client *kubernetes.Clientset) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		fmt.Println("graceful-shutdown keydb")
		for {
			current, err := isCurrentEndpoint(ctx, config, client)
			if !current && err == nil {
				break
			}
			if err != nil {
				fmt.Println(err)
			}
			time.Sleep(1 * time.Second)
		}
		w.Write([]byte("ok"))
		fmt.Println("can shutdown keydb")
	}
}

func gracefulShutdownManagerHandler(manager *Manager) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		fmt.Println("graceful-shutdown manager")
		for {
			err := manager.Check(r)
			if err != nil {
				break
			}
			time.Sleep(1 * time.Second)
		}
		w.Write([]byte("ok"))
		fmt.Println("can shutdown manager")
	}
}

func isCurrentEndpoint(ctx context.Context, config Config, client *kubernetes.Clientset) (bool, error) {
	endpoints, err := client.CoreV1().Endpoints(config.Namespace).Get(ctx, config.Instance+"-keydb", metav1.GetOptions{})
	if err != nil {
		if errors.IsNotFound(err) {
			return false, nil
		}
		return false, err
	}
	for _, subset := range endpoints.Subsets {
		for _, addresse := range subset.Addresses {
			if addresse.Hostname == config.Hostname {
				return true, nil
			}
		}
	}
	return false, nil
}
